/* find the capacitor ratios for each capacitor block for Adore2 */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* last change: 7/11/87 */

/*
 * Modifications to Adore
 * Henry Chang (henryc@ic.Berkeley.EDU)
 * Fall, 1990
 */

#define C_FILE_NAME "CBLKcapRatios.c"

#include "ADglobals.h"
#include <stdio.h>
#include <math.h>


/* the following also updates the topsum, botsum, and totalsum fields */
CBLKcapRatios()
{
  double mincapsize;
  double ceil();

  struct _module *tmpmod;
  struct _cblk *tmpcblk;
  struct _cap *tmpcap;


  for ( tmpmod = modlist[CBLK]; tmpmod != NULL; tmpmod = tmpmod->next ) {
    tmpcblk = tmpmod->info.cblk;

    /*** find the smallest capacitor in this block */
    mincapsize = MAXREAL;
    for (tmpcap= tmpcblk->contcaps; tmpcap != NULL; tmpcap = tmpcap->next) {
      if ( tmpcap->size <= 0 ) {
	printf("CBLKcapRatio: capacitor with non positive value\n");
	gerror = ON;
      }
      if ( tmpcap->size < mincapsize ) {
	mincapsize = tmpcap->size;
      }
    }
    for (tmpcap = tmpcblk->swcaps; tmpcap != NULL; tmpcap = tmpcap->next) {
      if ( tmpcap->size <= 0 ) {
	printf("CBLKcapRatio: capacitor with non positive value\n");
	gerror = ON;
      }
      if ( tmpcap->size < mincapsize ) {
	mincapsize = tmpcap->size;
      }
    }

    /*** the ratio is the size divided by the smallest unit value */
    tmpcblk->botsum = 0;
    for (tmpcap= tmpcblk->contcaps; tmpcap != NULL; tmpcap = tmpcap->next) {
      tmpcap->ratio = tmpcap->size / mincapsize;
      tmpcap->cratio = (int)ceil(tmpcap->ratio);

      tmpcblk->botsum += tmpcap->cratio;
    }

    tmpcblk->topsum = 0;
    for (tmpcap = tmpcblk->swcaps; tmpcap != NULL; tmpcap = tmpcap->next) {
      tmpcap->ratio = tmpcap->size / mincapsize;
      tmpcap->cratio = (int)ceil(tmpcap->ratio);

      tmpcblk->topsum += tmpcap->cratio;
    }
    tmpcblk->totalsum = tmpcblk->topsum + tmpcblk->botsum;
  }
}
