/* find the equivalent terminals of capacitor block for Adore2 */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* Last Change: 7/31/87 */

#include "CBLKglobals.h"
#include "TEKtech.h"
#include <stdio.h>

static void
  CBLKaddEqt( terminal, xcoord, ycoord )
struct _terminal *terminal;
int xcoord, ycoord;
{
    struct _eqterm *tmpeqt;
    
    
    CALLOC( tmpeqt , 1, struct _eqterm);
    tmpeqt->next = terminal->eqterms;
    terminal->eqterms = tmpeqt;
    
    tmpeqt->module = terminal->module;
    
    tmpeqt->xcoord = xcoord;
    tmpeqt->ycoord = ycoord;
    terminal->eqterms = tmpeqt;     /*  Enrico */
}

  CBLKfindEqt(cblk, flag)
struct _cblk *cblk;
int flag; /* whether HNUC condition should be checked or not */
{
    int numrow, numcol;
    int deltax, deltay; /* seperation of capacitor instances in x and y dim */
    int xcoord, ycoord;
    int yoffset;
    int type; /* capacitor type */
    struct _cap *tmpcap;


    deltax = cblk->unitw + 2 * botextop + botbotsp;
    deltay = cblk->unith + 2 * botextop + botbotsp;

    /* initialize the equivalent input lists */
    for ( tmpcap = cblk->contcaps; tmpcap != NULL; tmpcap = tmpcap->next ) {
	tmpcap->topplate->eqterms = tmpcap->botplate->eqterms = NULL;
    }
    for ( tmpcap = cblk->swcaps; tmpcap != NULL; tmpcap = tmpcap->next ) {
	tmpcap->topplate->eqterms = tmpcap->botplate->eqterms = NULL;
    }
    
    /* identify the equivalent inputs */
    
    /* all the topplates of the same input capacitors share the same memory */
    for ( numcol = 1; numcol <= cblk->numcol; numcol++) {
	xcoord = (numcol-1) * deltax; /*  + cblk->unitw/2 + botextop; */
	
	yoffset = 0;
	type = CONT;
	for ( numrow = 1; numrow <= cblk->numrow; numrow++ ) {
	    tmpcap = ucap_matrix[numrow][numcol];
	    if ( tmpcap == NULL ) {
		continue;
	    }
	    if ( tmpcap->type == type ) {
		ycoord = yoffset + (numrow - 1) * deltay;
		
		if ( flag == NO || ucap_flags[numrow][numcol] != HNUC ) {
		    CBLKaddEqt( tmpcap->topplate, xcoord+cblk->unitw/2+botextop, ycoord);
		}
		
		CBLKaddEqt( tmpcap->botplate, xcoord+polywidth/2 , ycoord );
		
		break;
	    }
	}
	
	yoffset = deltay - botbotsp; /* top of the unit capacitor */
	type = SWITCHED;
	for ( numrow = cblk->numrow; numrow >= 1; numrow-- ) {
	    tmpcap = ucap_matrix[numrow][numcol];
	    if ( tmpcap == NULL ) {
		continue;
	    }
	    if ( tmpcap->type == type ) {
		ycoord = yoffset + (numrow - 1) * deltay;
		
		if ( flag == NO || ucap_flags[numrow][numcol] != HNUC ) {
		    CBLKaddEqt( tmpcap->topplate, xcoord+cblk->unitw/2+botextop, ycoord);
		}
		
		CBLKaddEqt( tmpcap->botplate, xcoord+polywidth/2 , ycoord );
		
		break;
	    }
	}
    }
}
