/*
 * Modifications to Adore
 * Henry Chang (henryc@ic.Berkeley.EDU)   -  Fall, 1990
 */
#include <stdio.h>
#include "CBLKglobals.h"
#include "CBLKstr.h"
#include "TEKtech.h"

#define C_FILE_NAME "CBLKgenSymb.c"

struct _cap *ucap_matrix[MAXNUMROW][MAXNUMCOL]; /* symbolic matrix */

static void
  swap(coord1,coord2)
struct coord coord1, coord2;
{
    struct _cap *tmpcap;
    
    tmpcap = ucap_matrix[coord1.rownum][coord1.colnum];
    ucap_matrix[coord1.rownum][coord1.colnum] = ucap_matrix[coord2.rownum][coord2.colnum];
    ucap_matrix[coord2.rownum][coord2.colnum] = tmpcap;
}

static void
  mirrorx(botrow,toprow,leftcol,rightcol)
int botrow, toprow, rightcol, leftcol;
{
    int colcount, rowcount;
    struct coord coord1, coord2;
    
    for ( rowcount = botrow;rowcount<= toprow/2; rowcount++) {
	coord1.rownum = rowcount;
	coord2.rownum = toprow + 1 - rowcount;
	for ( colcount = leftcol;colcount <= rightcol; colcount++) {
	    coord1.colnum = coord2.colnum = colcount;
	    swap(coord1,coord2);
	}
    }
}

static void
  mirrory(botrow,toprow,leftcol,rightcol)
int botrow, toprow, rightcol, leftcol;
{
    int colcount, rowcount;
    struct coord coord1, coord2;
    
    for ( colcount = leftcol;colcount <= rightcol/2;colcount++) {
	coord1.colnum = colcount;
	coord2.colnum = rightcol + 1 - colcount;
	for ( rowcount = botrow;rowcount<= toprow; rowcount++) {
	    coord1.rownum = coord2.rownum = rowcount;
	    swap(coord1,coord2);
	}
    }
}

/* This procedure generates the symbolic placement of unit c's per cblks*/
/* This is the new improved version which doesnot fail */

CBLKgenSymb( cblk)
struct _cblk *cblk; /* The capacitor cblk to be placed */
{
  struct _cap *caplist[201];

  int ncaptop,ncapbot,count;	/* # of top and bottom capacitors */
  int topsum, botsum;		/* sum of the top and bottom capacitors per cblk */
  int totalsum;

  int height,width;		/* for each individual cblk in # of unit capacitors */ 
  int maxwidth, topwidth;	/* estimated width required for the cblk */
  int topheight;		/* hight of the top capacitors */
  int startcol;			/* for all cases it is 1 */
  double toph_r;		/* estimate for top height */
  double whitearea;		/* unused area due to zeros in matrix */
  double ptime(), stime, etime;
  double ceil();
  struct _cap *topcaps, *botcaps; /* temporary list of capaitors per cblk*/
  struct _cap *tempcap;		/* */

  stime = ptime();

  width = 0;
  height = cblk->numrow;
  topsum = cblk->topsum;
  botsum = cblk->botsum;
  totalsum = cblk->totalsum;
  ncaptop = cblk->nswitc;
  ncapbot = cblk->ncontc;
  topcaps = cblk->swcaps;
  botcaps = cblk->contcaps;

  /* evaluate an estimate of the height and width
     needed for the top capacitors */

  maxwidth = cblk->numcol;

  if ( cblk->module->transform == NO || 1 == 1 ) {
    /*** fill the caplist with the reverse order of topcaps */
    for ( tempcap = topcaps, count = ncaptop;tempcap != NULL, count >= 1; 
	 tempcap = tempcap->next, count-- ) {
      caplist[count] = tempcap;
    }

    /*** fill the caplist with botcaps */
    for ( tempcap = botcaps, count = ncaptop + 1;
	 tempcap != NULL, count <= ncaptop + ncapbot; 
	 tempcap = tempcap->next, count++ ) {
      caplist[count] = tempcap;
    }
  }
  else if ( cblk->module->transform == YES && 1 == 0 ) {
    /* for topcaps two mirrory operations cancel each other */
    for ( tempcap = topcaps, count = 1; tempcap != NULL, count <= ncaptop; 
	 tempcap = tempcap->next, count++ ) {
      caplist[count] = tempcap;
    }

    for ( tempcap = botcaps, count = ncaptop + ncapbot;
	 tempcap != NULL, count >= ncaptop + 1; 
	 tempcap = tempcap->next, count-- ) {
      caplist[count] = tempcap;
    }
  }


  startcol = 1;
  /* the following is used only to find necessary topheight and topwidth */

  topwidth = maxwidth + ncaptop/maxwidth - 1;
  while ( 1 ) {
    /*the following is used only to find necessary topheight and topwidth */
    topheight = (int)ceil((double)topsum/(double)topwidth);
    while( 1 ) {
      initialize(1,topheight, 1, topwidth);
      if (algo1(topheight,startcol,&width,1,ncaptop,caplist,
		&topwidth) == 0){
	topwidth++;
	if ( topwidth > maxwidth ) {
	  if ( topheight < height ) {
	    topheight++;
	    topwidth--;
	  }
	  else {
	    maxwidth = topwidth;
	  }
	}
      }
      else {
	break;
      }
    }
    initialize(1,height,1,maxwidth);
    algo1(topheight,startcol,&width,1,ncaptop,caplist,&topwidth) ;
    mirrorx(1,height,startcol,maxwidth);
    mirrory(1,height,startcol,maxwidth);
	

    if (algo1(height,startcol,&width,ncaptop+1,ncaptop+ncapbot,caplist,
	      &maxwidth) == 0 ) {;
				 if ( topwidth < maxwidth ) {
				   topwidth = maxwidth;
				 }
				 else {
				   maxwidth++;
				   topwidth = maxwidth + ncaptop/maxwidth - 1;
				 }
			       }
    else {
      break;
    }
  }

  if ( cblk->module->transform == YES && 1 == 0 ) {
    mirrory(1,height,startcol,width);
  }

  cblk->numcol = maxwidth;
  cblk->module->xdim = cblk->numcol *(cblk->unitw + 2 * botextop + botbotsp)
    - botbotsp;

  etime = ptime();
}

algo1(height,startcol,pwidth,firstcap,lastcap,caplist,pmaxwidth)
int height,startcol,*pwidth,firstcap,lastcap;
struct _cap *caplist[];
int *pmaxwidth; /* estimated value of width required */
{
  /* the algorithm to place capacitors in an array
     of height h such that each capacitor has a terminal
     at the bottom of the array */

  int    count, size;
  int    wflag;			/* whehter the maximum width has been increased */
  struct neighbor *nbr;
  struct coord curloc;

  for ( count = firstcap; count <= lastcap ; count++ ) {
    size = 1;
    /* assign the leftmost unoccupied location
       of the first row to the current capacitor */

    /* reset the neighborhood list */
		
    nbr = NULL;

    if ( putfirst(caplist[count],&curloc,&pwidth,startcol,*pmaxwidth) ==0) {
      /* need to increase width */
      return( 0 );
    }

    while ( size < caplist[count]->cratio ) {
      size++;

      /* update all the unoccupied neighboring
	 unit cells of the newly added unit cell */

      updatnbr2(curloc,&nbr,height,startcol,*pmaxwidth);

      /* grow the current capacitor by one */

      if ( growunit(caplist[count],&nbr,&curloc,&pwidth) == 0 ) {
	/* need to increase width */
	return ( 0 );
      }
    }
  }
  return ( 1 );
}

initialize(botheight,topheight,leftwidth,rightwidth)
int botheight,topheight,leftwidth,rightwidth;
{
  int temprow,tempcol;

  for (temprow=topheight;temprow>=botheight;temprow--) {
    for (tempcol=rightwidth;tempcol>=leftwidth;tempcol--) {
      ucap_matrix[temprow][tempcol] = NULL;
    }
  }
}

putfirst(capacitor,location,ppwidth,startcol,maxwidth)
struct _cap *capacitor;
struct coord *location;
int **ppwidth,startcol;
int maxwidth;
{
  int temp;

  /* find the leftmost unoccupied location */
  for ( temp = startcol;temp <= maxwidth; temp++ ) {
    if (ucap_matrix[1][temp] == NULL) {
      ucap_matrix[1][temp] = capacitor;
      location->rownum = 1;
      location->colnum = temp;

      if ( temp > **ppwidth )
	**ppwidth  = temp;

      break;
    }
  }
  if ( temp > maxwidth ) {
    return ( 0 );
  }
  return ( 1 );
}

updatnbr(location,list,height,startcol)
struct coord location;
struct neighbor **list;
int height,startcol;
{
  struct coord temploc;

  if ( location.colnum == MAXNUMCOL - 1 ) {
    printf("need to increase column size of array\n");
    exit(2);
  }
  else {
    if ( location.colnum > startcol ) {
      /* check left neighbor */
      if ( ucap_matrix[location.rownum][location.colnum-1]== NULL ) {
	temploc.rownum = location.rownum;
	temploc.colnum = location.colnum-1;
	addnbr(temploc,&list);
      }
    }
    if ( location.rownum > 1 ) {
      /* check bottom neighbor */
      if ( ucap_matrix[location.rownum-1][location.colnum]== NULL ) {
	temploc.rownum = location.rownum-1;
	temploc.colnum = location.colnum;
	addnbr(temploc,&list);
      }
    }
    if ( location.rownum < height ) {
      /* check top neighbor */
      if ( ucap_matrix[location.rownum+1][location.colnum]== NULL ) {
	temploc.rownum = location.rownum+1;
	temploc.colnum = location.colnum;
	addnbr(temploc,&list);
      }
    }
    /* check right neighbor */
    if ( ucap_matrix[location.rownum][location.colnum+1]== NULL ) {
      temploc.rownum = location.rownum;
      temploc.colnum = location.colnum+1;
      addnbr(temploc,&list);
    }
  }
}

/* this is the modified update neighborhood procedure. When updating
	the right neighbor, the right bound is given by the estimate of
	the width needed to fit all the capacitors in the cblk
*/

updatnbr2(location,list,height,startcol,maxwidth)
struct coord location;
struct neighbor **list;
int height,startcol;
int maxwidth;
{
  struct coord temploc;

  if ( location.colnum == MAXNUMCOL - 1 ) {
    printf("need to increase column size of array\n");
    exit(2);
  }
  else {
    if ( location.colnum > startcol ) {
      /* check left neighbor */
      if ( ucap_matrix[location.rownum][location.colnum-1]== NULL ) {
	temploc.rownum = location.rownum;
	temploc.colnum = location.colnum-1;
	addnbr(temploc,&list);
      }
    }
    if ( location.rownum > 1 ) {
      /* check bottom neighbor */
      if ( ucap_matrix[location.rownum-1][location.colnum]== NULL ) {
	temploc.rownum = location.rownum-1;
	temploc.colnum = location.colnum;
	addnbr(temploc,&list);
      }
    }
    if ( location.rownum < height ) {
      /* check top neighbor */
      if ( ucap_matrix[location.rownum+1][location.colnum]== NULL ) {
	temploc.rownum = location.rownum+1;
	temploc.colnum = location.colnum;
	addnbr(temploc,&list);
      }
    }
    if ( location.colnum < maxwidth ) {
      /* check right neighbor */
      if ( ucap_matrix[location.rownum][location.colnum+1]== NULL ) {
	temploc.rownum = location.rownum;
	temploc.colnum = location.colnum+1;
	addnbr(temploc,&list);
      }
    }
  }
}

/* This procedure adds the unoccupied neigbor "place" to the list of
	unoccupied neighbors plist 
*/

addnbr(place,plist)
struct coord place;
struct neighbor ***plist;
{
  char *calloc();
  struct neighbor *tempptr,*auxptr;
  int flag;

  flag = 0;
  tempptr = (struct neighbor *) calloc(1,sizeof(struct neighbor));

  tempptr->coord.rownum = place.rownum;
  tempptr->coord.colnum = place.colnum;
  tempptr->next = NULL;

  if ( **plist == NULL )
    **plist = tempptr;

  else {
    /* place first according to column order */
    tempptr->next = **plist;
    while ( ( tempptr->next!=NULL ) && 
	   (tempptr->coord.colnum > tempptr->next->coord.colnum)){
      auxptr = tempptr->next;
      tempptr->next = tempptr->next->next;
    }

    /* now place according to row order */
    while ((tempptr->next!=NULL) && 
	   (tempptr->coord.colnum==tempptr->next->coord.colnum)
	   && (tempptr->coord.rownum >=
	       tempptr->next->coord.rownum)) {

      /* the cell already is in the neighbor list */
      if (tempptr->coord.rownum ==
	  tempptr->next->coord.rownum ) {
	flag = 1;
	break;
      }

      auxptr = tempptr->next;
      tempptr->next = tempptr->next->next;
    }
    if (flag == 0 ) {
      if ( tempptr->next == **plist )
	**plist = tempptr;
      else 
	auxptr->next = tempptr;
    }
  }
}

growunit(capacitor,list,location,ppwidth)
struct _cap *capacitor;
struct neighbor **list;
struct coord *location;
int **ppwidth;
{
  struct neighbor *auxptr,*preptr;
  int maxrownum;	/* maximum row number of any unoccpied neighbor */

  /* cmin is the column number of the leftmost unoccupied neighbors */

  if (*list == NULL) {
    return ( 0 );
  }

  if ( ucap_matrix[1][(*list)->coord.colnum] != NULL ) {

    /* the cell of coordintes (1,cmin) is occupied.
       in the column cmin, assign the unoccupied
       neighbor of lowest row number to the current capacitor*/

    ucap_matrix[(*list)->coord.rownum][(*list)->coord.colnum] = capacitor;


    if ( (*list)->coord.colnum > **ppwidth )
      /* it seems that this should not happen */
      **ppwidth = (*list)->coord.colnum;

    location->rownum = (*list)->coord.rownum;
    location->colnum = (*list)->coord.colnum;

    /* remove the assigned cell from neighbor list */

    *list = (*list)->next;
  }

  else {
    /* in the column cmin find the unoccupied
       neighbor of highest row number         */

    /*
      auxptr = *list;
      while ((auxptr->next != NULL) && 
      ( auxptr->coord.colnum == auxptr->next->coord.colnum)) {
      preptr = auxptr;
      auxptr = auxptr->next;
      }
      */
    /* find the unoccupied neighbor of heighest row number, among
       those choose the one with lowest column number
       */

    auxptr = *list;
    /* 
     * preptr == NULL; 
     *
     * Could this have been the error giving:
     * pid 2370 (adore2) was killed on unaligned access, at pc 0x404e94
     * causing it run on vaxes, but not on mips machines?
     * It seems like it.
     * -hcc
     */
    preptr = NULL;
    maxrownum = auxptr->coord.rownum;
    /* since the neigbors are ordered in the increasing column number,
       the first neighbor of highest row number has the lowest
       column number */
    while (auxptr->next != NULL)  { 
      if ( auxptr->next->coord.rownum > maxrownum) {
	preptr = auxptr;
	maxrownum = auxptr->next->coord.rownum;
      }
      auxptr = auxptr->next;
    }
    if (preptr == NULL) {
      auxptr = *list;
    }
    else {
      auxptr = preptr->next;
    }

    ucap_matrix[auxptr->coord.rownum][auxptr->coord.colnum] = capacitor;

    if ( auxptr->coord.colnum > **ppwidth )
      **ppwidth = auxptr->coord.colnum;

    location->rownum = auxptr->coord.rownum;
    location->colnum = auxptr->coord.colnum;

    /* remove the assigned cell from neighbor list */
    if ( auxptr == *list )
      *list = (*list)->next;
    else
      preptr->next = auxptr->next;
  }
  return ( 1 );
}
