/* this generates the actual layout from symbolic capacitor information
for the automatic generation of switched-capacitor circuits */

/* Author: Hormoz Yaghutiel   Advisor: Alberto Sangiovanni-Vincentelli
 * last change : 8/10/87
 * Henry Chang (henryc@ic.Berkeley.EDU) - Fall, 1990
 * Last update:  March 1992  -  Enrico Malavasi

/* defined: CBLKsymb2lay, CBLKnonucName, CBLKmkName */

#define C_FILE_NAME "CBLKsymb2lay.c"

#include <stdio.h>
#include <string.h>
#include "oct.h"
#include "oh.h"
#include "utility.h"
#include "CBLKglobals.h"
#include "CBLKstr.h"
#include "TEKtech.h"

char *CBLKmkName();
char *CBLKpushDir();
int ucap_flags[MAXNUMROW][MAXNUMCOL];

static int capresol; /* resolution of edges of the capacitors */

char *layer; /* name of a given layer */
int width; /* width of above layer */
int spacing; /* spacing of two elements of the same layer */

static int unitHeight, unitWidth;
static int arrayHeight, arrayWidth;
int deltax, deltay;

static octObject array_contents;
extern octObject *makebox();


static void
  CBLKstripDir(name)
char *name;
{
    char *i, *result;
    
    result = name;
    for (i=name; *i!=0; i++) {
	if (*i=='/') 
	  result = i+1;
    }
    strcpy(name, result);
}
static char *
  CBLKmkNonucap(capacitor, flag, instances)
struct _cap *capacitor;
int flag; /* the master of a horizontal(HNUC) or vertical(VNUC) instance */
char *instances;
/* Generate an instance of non unit capacitor */
{
    octObject nonuc_contents;
    octObject box;
    
    int nucWidth, nucHeight;
    int lowleftx, lowlefty, uprightx, uprighty;
    int slowleftx, slowlefty, suprightx, suprighty;
    int nlowleftx, nlowlefty;	/* temporary variable */
    int fracside, flr_fracside;	/*fractional part of non units, and the multiple
				  of resolution which fits in it */
    int tab_length;		/* length of the tab */
    double tab_area;		/* area of the tab needed because of rounding effects */
    double rint(), ceil(), fraction;
    char *CBLKnonucName();
    char *nucdirname, nucname[50];


    if ( flag == VNUC ) {
	nucWidth = unitWidth; nucHeight = unitHeight;
    }
    else if ( flag == HNUC ) {
	nucHeight = unitWidth; nucWidth = unitHeight;
    }
    
    /*** define contents of non unit capacitor */
    strcpy(nucname , CBLKnonucName( capacitor ));
    nucdirname = strdup(CBLKpushDir(instances, nucname));

    openfacet(&nonuc_contents,nucdirname, "physical","contents","w");
    
    fraction = capacitor->ratio - (int)capacitor->ratio;
    
    fracside = (int) rint(fraction * nucHeight);
    
    /*** find what multiple of capresol, fracside is */
    flr_fracside = Round_Int(fracside,capresol, FLR);
    
    /*** find the total length of the tabs */
    tab_area =(double)nucWidth * (fracside - flr_fracside);
    tab_length = (int) rint( tab_area / (double) capresol);
    tab_length = Round_Int(tab_length,capresol, RND);
    
    /*** make the fractional part of top plate */
    slowleftx = lowleftx = botextop;
    slowlefty = lowlefty =  - (botextop + botbotsp);
    suprightx = uprightx = lowleftx + nucWidth;
    suprighty = uprighty = lowlefty + flr_fracside;
    box = *makebox(&nonuc_contents,toppoly_L,lowleftx,lowlefty,uprightx,uprighty);
    
    if ( uctype == 1 ) {
	/*** make the metal covering the topplate */
	lowleftx = slowleftx + botextop;
	lowlefty = slowlefty - botextop;
	uprightx = suprightx - botextop;
	uprighty = suprighty - botextop;
	box = *makebox(&nonuc_contents, metal_L, lowleftx, lowlefty,uprightx, uprighty);
	
	/*** make the metal extension for topplate connection */
	lowleftx = slowleftx + (nucWidth - metwidth)/2;  LAMBDA_ROUND(lowleftx);
	lowlefty = suprighty - botextop;                 LAMBDA_ROUND(lowlefty);
	uprightx = lowleftx + metwidth;
	uprighty = nucHeight + botextop - botextop;      LAMBDA_ROUND(uprighty);
	box = *makebox(&nonuc_contents,metal_L,lowleftx,lowlefty, uprightx,uprighty);
    } else if ( uctype == 2 ) {
	/* now make the extension of top plate to the top */
	lowleftx = slowleftx + (nucWidth - polywidth)/2;
	lowlefty = suprighty;
	uprightx = lowleftx + polywidth;
	uprighty = nucHeight + botextop;
	box = *makebox(&nonuc_contents,toppoly_L,lowleftx,lowlefty, uprightx,uprighty);
    }

    /* add two tabs of height capresol on a fractional topplate of height flr_fracside */
    if ((2 * (fracside - flr_fracside))  <= capresol ) {
	/*** place two tabs of equal length on the two sides */
	lowleftx = slowleftx;
	lowlefty = suprighty;
	uprightx = lowleftx + Round_Int(tab_length/2, capresol, FLR);
	uprighty = suprighty + capresol;
	box = *makebox(&nonuc_contents, toppoly_L, lowleftx, lowlefty, uprightx, uprighty);
	lowleftx = suprightx - Round_Int( tab_length/2, capresol, CEIL );
	uprightx = suprightx;
	box = *makebox(&nonuc_contents, toppoly_L, lowleftx, lowlefty, uprightx, uprighty);
    } else {
	/* take out two tabs of height capresol from a fractional topplate
	   of height ceil_fracside and place one tab in the middle */
	lowleftx = Round_Int( (slowleftx + suprightx - tab_length)/2,capresol, RND);
	lowlefty = suprighty;
	uprightx = lowleftx + tab_length;
	uprighty = lowlefty + capresol;
	box = *makebox(&nonuc_contents, toppoly_L, lowleftx, lowlefty, uprightx, uprighty);
	suprighty += capresol;
    }

    /*** make the bottom poly extension over fraction */
    lowleftx = slowleftx - botextop;
    lowlefty = slowlefty + botextop;
    uprightx = suprightx + botextop;
    uprighty = suprighty + botextop;
    box = *makebox(&nonuc_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
    
    /*** now make two boxes for extensions of bottom plate */
    
    lowleftx = slowleftx - botextop;
    /*** the following botextop is actual spacing and not overlap*/
    /*** is used for both metal and poly */
    uprightx = slowleftx + (nucWidth - width)/2 - botextop;
    lowlefty = suprighty + botextop;
    uprighty = nucHeight + 2 * botextop;
    box = *makebox(&nonuc_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
    
    lowleftx = uprightx + 2 * botextop + width;
    uprightx = suprightx + botextop;
    box = *makebox(&nonuc_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
    
    OH_ASSERT(octCloseFacet(&nonuc_contents));
    return nucdirname;
}

static void
  CBLKplaceNonucaps(instances)
char *instances;
/* place all the non unit capacitors */
{
    octTransformType transform;

    int tempcol;
    int xtrans, ytrans;
    double floor();
    char name[100];
    
    struct _cap *capacitor;
    
    
    for (tempcol = 1; tempcol <= arrayWidth; tempcol++) {
	
	capacitor = ucap_matrix[1][tempcol];
	
	if ( capacitor != NULL && capacitor->nuc_placed == NO ) {
	    if ( capacitor->ratio - floor( capacitor->ratio ) < 1.e-4) {
		capacitor->nuc_placed = YES;
	    } else {
		if ( arrayHeight > 1 && ucap_matrix[2][tempcol] == capacitor ){
		    
		    strcpy(name , CBLKmkNonucap(capacitor, VNUC, instances));
		    
		    /* translate the created instance in the x and y directions */
		    xtrans = (tempcol - 1) * deltax;
		    ytrans = deltay;
		    transform = OCT_NO_TRANSFORM;
		    /*
		     * The directory information seems to be already known by OCT
		     * Therefore, only want the filename.
		     */
		    CBLKstripDir (name);  
		    makeinstance(&array_contents,name, name,"physical",
				 transform, xtrans, ytrans);
		    capacitor->nuc_placed = YES;
		    ucap_flags[2][tempcol] = VNUC;
		}
		
		else if (tempcol < arrayWidth &&
			 ucap_matrix[1][tempcol+1] == capacitor) {
		    
		    /*** the order of unitHeight and unitWidth is switched
		      in the following because the instance has a 90d rotation */
		    strcpy(name , CBLKmkNonucap(capacitor, HNUC, instances));
		    
		    /* rotate 90 degrees */
		    /* translate the created instance in the x and y
		       directions */
		    xtrans = tempcol * deltax;
		    ytrans = 0;
		    transform = OCT_MY_ROT90;
		    CBLKstripDir (name);  
		    makeinstance(&array_contents,name, name,"physical",
				 transform, xtrans, ytrans);
		    
		    capacitor->nuc_placed = YES;
		    ucap_flags[1][tempcol + 1] = HNUC;
		    
		} else {
		    fprintf(stderr,"CBLKplaceNonucaps;1: internal error\n");
		    exit(-1);
		}
	    }
	}

	/* now do the same for capacitors having terminals 
	   on top of the array */
	
	capacitor = ucap_matrix[arrayHeight][tempcol];
	
	if ( capacitor != NULL && capacitor->nuc_placed == NO ) {
	    if ( capacitor->ratio - floor( capacitor->ratio ) < 1.e-4) {
		capacitor->nuc_placed = YES;
	    } else {
		
		if ( arrayHeight > 1 && 
		    ucap_matrix[arrayHeight-1][tempcol]== capacitor) {
		    strcpy(name , CBLKmkNonucap(capacitor, VNUC, instances));
		    
		    /* rotate the instance about x axis */
		    /* translate the created instance in the x and y
		       directions */
		    xtrans = (tempcol - 1) * deltax;
		    ytrans = (arrayHeight - 2) * deltay;
		    transform = OCT_MIRROR_Y;
		    /* because of mirroring we have to move the instance */
		    ytrans += deltay - botbotsp;
		    CBLKstripDir (name);  
		    makeinstance(&array_contents,name, name,"physical",
				 transform, xtrans, ytrans);
		    
		    capacitor->nuc_placed = YES;
		    ucap_flags[arrayHeight-1][tempcol] = VNUC;
		} else if (tempcol < arrayWidth && 
			   ucap_matrix[arrayHeight][tempcol+1] == capacitor) {
		    /*** the order of unitHeight and unitWidth is switched
		      in the following because the instance has a 90d rotation */
		    strcpy(name , CBLKmkNonucap(capacitor, HNUC, instances));
		    
		    /* rotate 90 degrees */
		    /* translate the created instance in the x and y
		       directions */
		    xtrans = tempcol * deltax;
		    ytrans = (arrayHeight -1 ) * deltay;
		    transform = OCT_MY_ROT90;
		    CBLKstripDir (name);  
		    makeinstance(&array_contents,name, name,"physical",
				 transform, xtrans, ytrans);
		    
		    capacitor->nuc_placed = YES;
		    ucap_flags[arrayHeight][tempcol + 1] = HNUC;
		}
		
		else {
		    printf("error in creating nonunit parts\n");
		    exit(-1);
		}
	    }
	}
    }
}

/* make the metal (poly) tab used for top plate connections */
static void
  CBLKtopplateTab( rownum, colnum, sideflag, shiftflag, longTab )
int rownum, colnum;
int sideflag, shiftflag;
int longTab;
{
    octObject box;
    int lowleftx, lowlefty, uprightx, uprighty;
    int xtrans, ytrans;

    xtrans = ( colnum - 1 ) * deltax;
    ytrans = ( rownum - 1 ) * deltay;

    switch( sideflag ) {
      case TOP:
	lowleftx = botextop + (unitWidth - width)/2;
	lowlefty = botextop + unitHeight;           
	uprightx = lowleftx + width;
	uprighty = lowlefty + botextop + Round_Int(botbotsp/2, capresol, CEIL);
	if ( uctype == 1 ) {
	    lowlefty -= botextop; 
	}
	if ( shiftflag == YES ) {
	    lowleftx -= Round_Int((spacing+width) /2, capresol, CEIL );
	    uprightx -= Round_Int((spacing+width) /2, capresol, CEIL );
	}
	if((longTab == YES) && (rownum < arrayHeight )) {
	    uprighty += (deltay/2); /*  Longer tabs are needed for connections with fractional caps */
	}
	break;
	
      case BOTTOM:
	lowleftx = botextop + (unitWidth - width)/2;
	lowlefty = - Round_Int(botbotsp/2, capresol, CEIL);
	uprightx = lowleftx + width;
	uprighty = botextop;
	if ( uctype == 1 ) {
	    uprighty += botextop; 
	}
	if ( shiftflag == YES ) {
	    lowleftx += Round_Int((spacing+width)/2, capresol, CEIL );
	    uprightx += Round_Int((spacing+width)/2, capresol, CEIL );
	}
	if( (longTab == YES) && ( rownum > 1 )) {
	    lowlefty -= (deltay/2); /*  Longer tabs are needed for connections with fractional caps */
	}
	break;
	
      case RIGHT:
	lowleftx = botextop + unitWidth;
	lowlefty = botextop + (unitHeight - width)/2;
	uprightx = lowleftx + botextop + Round_Int(botbotsp/2, capresol, CEIL);
	uprighty = lowlefty + width;
	if ( uctype == 1 ) {
	    lowleftx -= botextop; 
	}
	if ( shiftflag == YES ) {
	    lowlefty += Round_Int((spacing+width)/2, capresol, CEIL );
	    uprighty += Round_Int((spacing+width)/2, capresol, CEIL );
	}
	if((longTab == YES) && (colnum < arrayWidth )) {
	    uprightx += (deltax/2); /*  Longer tabs are needed for connections with fractional caps */
	}
	break;
	
      case LEFT:
	lowleftx = - Round_Int(botbotsp / 2, capresol, CEIL);
	lowlefty = botextop + (unitHeight - width)/2;
	uprightx = lowleftx + botextop + Round_Int(botbotsp/2, capresol, CEIL);
	uprighty = lowlefty + width;
	if ( uctype == 1 ) {
	    uprightx += botextop; 
	}
	if ( shiftflag == YES ) {
	    lowlefty -= Round_Int((spacing+width)/2, capresol, CEIL );
	    uprighty -= Round_Int((spacing+width)/2, capresol, CEIL );
	}
	if( (longTab == YES) && ( colnum > 1 )) {
	    lowleftx -= (deltax/2); /*  Longer tabs are needed for connections with fractional caps */
	}
	break;
    }
    
    lowleftx += xtrans;    LAMBDA_ROUND(lowleftx);
    lowlefty += ytrans;    LAMBDA_ROUND(lowlefty);
    uprightx += xtrans;    LAMBDA_ROUND(uprightx);
    uprighty += ytrans;    LAMBDA_ROUND(uprighty);
    
    box = *makebox(&array_contents,layer,lowleftx,lowlefty, uprightx,uprighty);
}

static void
  CBLKtopConnect(rownum, colnum)
int rownum, colnum;
{
    int i, sideflag;
    struct _cap *curruc, *nextuc;
    
    for(i=0 ; i<4 ; i++) {
	if(i == 0) {
	    sideflag = BOTTOM;
	    nextuc = ( rownum == 1 ) ? NULL : ucap_matrix[rownum - 1][colnum];
	}
	if(i == 1) {
	    sideflag = TOP;
	    nextuc = ( rownum == arrayHeight ) ? NULL : ucap_matrix[rownum + 1][colnum];
	}
	if(i == 2) {
	    sideflag = LEFT;
	    nextuc = ( colnum == 1 ) ? NULL : ucap_matrix[rownum][colnum - 1];
	}
	if(i == 3) {
	    sideflag = RIGHT;
	    nextuc = ( colnum == arrayWidth ) ? NULL :ucap_matrix[rownum][colnum + 1];
	}

	curruc = ucap_matrix[rownum][colnum];
	if ( nextuc == NULL || 
	    curruc->topplate->netptr == nextuc->topplate->netptr) {
	    
	    /* curruc and nextuc have the same input phase
	     * or the neighbor is empty */
	    
	    /* make the top plate connections for unit capacitors and put in center */
	    if ( ( (sideflag == TOP || sideflag == BOTTOM) &&
		  ucap_flags[rownum][colnum] == HNUC) ||
		( (sideflag == RIGHT || sideflag == LEFT) &&
		 ucap_flags[rownum][colnum] == VNUC ) ) {
		/* can't put any tabs */
	    } else {
		CBLKtopplateTab(rownum, colnum, sideflag, NO, (nextuc == NULL)?NO:YES);
	    }
	} else if ( ucap_flags[rownum][colnum] != HNUC &&
		   ucap_flags[rownum][colnum] != VNUC ){
	    /* add equal metal to botplate parasitics to all units */
	    /* this generates parasitics equal to what is generated
	       by metal overlap of botplate for metal connections */
	    /*** shift the tab */
	    
	    CBLKtopplateTab(rownum, colnum, sideflag, YES, NO);
	}
    }
}

/* generate the unit capacitor for this capacitor block */
static void
  CBLKmkUcap( cblk, instances )
struct _cblk *cblk;
char *instances;
{
    octObject unitcap_contents;
    octObject box;
    int lowleftx, lowlefty, uprightx, uprighty;
    
    char ucdirname[100], ucname[50];
    
    /*** define contents of unit capacitor */
    strcpy(ucname , CBLKmkName(cblk->opamp->name, cblk->in_polarity, "UCAP"));
    strcpy(ucdirname, CBLKpushDir(instances, ucname));
    openfacet(&unitcap_contents, ucdirname,"physical","contents","w");

    /*** make the bottom plate */
    lowleftx = 0;
    lowlefty = 0;
    uprightx = unitWidth + 2 * botextop;
    uprighty = unitHeight + 2 * botextop;
    box = *makebox(&unitcap_contents, botpoly_L, lowleftx, lowlefty,uprightx, uprighty);
    
    /*** make the poly top plate */
    lowleftx += botextop;
    lowlefty += botextop;
    uprightx -= botextop;
    uprighty -= botextop;
    box = *makebox(&unitcap_contents, toppoly_L, lowleftx, lowlefty,uprightx, uprighty);
    
    /*** make the contact */
    if ( uctype == 1 ) {
	/*** The metal plate covers almost the whole area of top plate */
	lowleftx += botextop;
	lowlefty += botextop;
	uprightx -= botextop;
	uprighty -= botextop;
	box = *makebox(&unitcap_contents, metal_L, lowleftx, lowlefty, uprightx, uprighty);
	/*** Contact box in the middle of the top plate */
	lowleftx = (unitWidth - mincontd)/2 + botextop;
	lowlefty = (unitHeight - mincontd)/2 + botextop;
	uprightx = lowleftx + mincontd;
	uprighty = lowlefty + mincontd;
	box = *makebox(&unitcap_contents,cop2_L, lowleftx,lowlefty,uprightx,uprighty);
    } else if ( uctype == 2 ) {
	/*** no contact needed */
    } else {
	errRaise(C_FILE_NAME, -1, "CBLKmkUcap;1: internal error");
    }
    
    OH_ASSERT(octCloseFacet(&unitcap_contents));
}

static void
  CBLKplaceUcaps( cblk, instances )
struct _cblk *cblk;
char *instances;
/* Place the unit capacitors and generate the bottom plate connection
 * of unit capacitors */
{
    octObject box;
    
    int incount;
    int temprow,tempcol;
    int lowleftx, lowlefty, uprightx, uprighty;
    int firstc, lastc;		/* start and stop x postions for bottom plate strips*/
    
    char ucname[100], ucdirname[100];
    
    struct _cap *capacitor;
    struct _cap *curruc, *nextuc; /* the array name of two adjacent unit caps
				     to be compared for making connections */
    
    
    CBLKmkUcap( cblk, instances );
    
    strcpy(ucname , CBLKmkName(cblk->opamp->name, cblk->in_polarity, "UCAP"));
    strcpy(ucdirname, CBLKpushDir(instances, ucname));
    
    for (temprow=1;temprow<=arrayHeight;temprow++) {
	
	/* the bottom plate connections in the y direction are generated
	   in long strips, the following variables indicate the  
	   starting and stoping x coordinates of these strips */
	firstc = lastc = 0;
	
	for (tempcol=1;tempcol<=arrayWidth;tempcol++) {
	    
	    curruc = ucap_matrix[temprow][tempcol];
	    if ( curruc == NULL ) {
		/* The following has to be updated */
		lastc = firstc = tempcol;
		ucap_flags[temprow][tempcol] = PUT;
		
		continue;
	    }
	    
	    if ( ucap_flags[temprow][tempcol] == UNPUT ) {
		CBLKstripDir (ucdirname);
		makeinstance(&array_contents,ucname, ucdirname, "physical",
			     OCT_NO_TRANSFORM, (tempcol-1)*deltax, (temprow-1)*deltay);
		ucap_flags[temprow][tempcol] = PUT;
	    }
	    
	    /*** make the top plate connections for unit capacitors */
	    
	    /*** check the four directions */
	    CBLKtopConnect(temprow, tempcol);
	    
	    /* now make the bottom plate unit capacitor connections */
	    if (temprow < arrayHeight) { 
		nextuc = ucap_matrix[temprow + 1][tempcol];
		
		if (curruc == nextuc) {
		    /* increase the length of y direction strip by 1 */
		    lastc++;
		}
		else {
		    if ( lastc > firstc ) {
			/* make an extended y direction connection */
			lowleftx = firstc * deltax;
			lowlefty = temprow * deltay - botbotsp;
			uprightx = lowleftx + botextop + (unitWidth -
							  width) / 2 - botextop;
			uprighty = lowlefty + botbotsp;
			box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
			for (incount = 1; incount < lastc - firstc; incount++){
			    lowleftx = uprightx + 2 * botextop + width;
			    uprightx = lowleftx + deltax - 2* botextop - width;
			    box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
			}
			lowleftx = uprightx + 2 * botextop + width;
			uprightx = lastc * deltax - botbotsp;
			box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
		    }
		    lastc = firstc = tempcol;
		}
	    }
	    
	    if (tempcol < arrayWidth) {  
		nextuc = ucap_matrix[temprow][tempcol + 1];
		
		if (curruc == nextuc) {
		    /* make the x direction connections for bottom plate */
		    lowleftx = tempcol * deltax - botbotsp;
		    lowlefty = (temprow - 1 ) * deltay;
		    uprightx = lowleftx + botbotsp;
		    uprighty = lowlefty + botextop + (unitHeight - width)/2
		      - botextop;
		    box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty,uprightx,uprighty);
		    lowlefty = uprighty + 2 * botextop + width;
		    uprighty = temprow * deltay - botbotsp;
		    box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty,uprightx,uprighty);
		} else {
		    if ( lastc > firstc ) {
			/* make an extended y direction connection */
			lowleftx = firstc * deltax;
			lowlefty = temprow * deltay - botbotsp;
			uprightx = lowleftx + botextop + (unitWidth -
							  width) / 2 - botextop;
			uprighty = lowlefty + botbotsp;
			box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
			for (incount = 1; incount < lastc - firstc; incount++){
			    lowleftx = uprightx + 2 * botextop + width;
			    uprightx = lowleftx+ deltax - 2 * botextop - width;
			    box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
			}
			lowleftx = uprightx + 2 * botextop + width;
			uprightx = lastc * deltax - botbotsp;
			box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
		    }
		    lastc = firstc = tempcol;
		}
	    } else if ( tempcol == arrayWidth ) {
		if ( lastc > firstc ) {
		    /* make an extended y direction connection */
		    lowleftx = firstc * deltax;
		    lowlefty = temprow * deltay - botbotsp;
		    uprightx = lowleftx + botextop + (unitWidth -
						      width) / 2 - botextop;
		    uprighty = lowlefty + botbotsp;
		    box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty,uprightx,uprighty);
		    for ( incount = 1; incount < lastc - firstc; incount++){
			lowleftx = uprightx + 2 * botextop + width;
			uprightx = lowleftx + deltax - 2 * botextop - width;
			box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty, uprightx,uprighty);
		    }
		    lowleftx = uprightx + 2 * botextop + width;
		    uprightx = lastc * deltax - botbotsp;
		    box = *makebox(&array_contents,botpoly_L,lowleftx,lowlefty,uprightx,uprighty);
		}
	    }
	}
    }
}

CBLKsymb2lay(cblk, instances)
struct _cblk *cblk;
char *instances;
{
    int cnt1, cnt2;
    char *cblkdirname, *cblkname;
    
    
    octBegin();
    
    if ( uctype == 1 ) {
	width = metwidth;
	layer = metal_L;
	spacing = metmetsp;
    }
    else if ( uctype == 2 ) {
	width = polywidth;
	layer = toppoly_L;
	spacing = polypolysp;
    }
    
    capresol = resolution / 2;
    
    unitHeight = cblk->unith;
    unitWidth = cblk->unitw;
    
    arrayHeight = cblk->numrow;
    arrayWidth = cblk->numcol;
    
    /* separation of capacitor instances in the x and y directions */
    deltax = unitWidth + 2 * botextop + botbotsp;
    deltay = unitHeight + 2 * botextop + botbotsp;
    
    /* initialize flags */
    for (cnt1 = 1; cnt1 <= arrayHeight; cnt1++) {
	for ( cnt2 = 1; cnt2 <= arrayWidth; cnt2++ ) {
	    ucap_flags[cnt1][cnt2] = UNPUT;
	}
    }
    
    cblkname    = strdup(CBLKmkName(cblk->opamp->name, cblk->in_polarity, "CBLK"));
    cblkdirname = strdup(CBLKpushDir(instances, cblkname));
    cblk->module->name = util_strsav( cblkdirname);

    openfacet(&array_contents,cblkdirname,"physical","contents","w");

    CBLKplaceNonucaps( instances );
    CBLKplaceUcaps( cblk, instances );
    
    OH_ASSERT(octCloseFacet(&array_contents));
    octEnd();
}


/* generate an oct directory pointer for a non_unit capacitor */
char *
  CBLKnonucName ( capacitor )
struct _cap *capacitor;
{
    char *tmpname;
    char *nonucname;
    
    tmpname = strdup(CBLKmkName(capacitor->cblk->opamp->name, capacitor->cblk->in_polarity, "NUC"));
    nonucname = (char *) calloc(strlen(tmpname) + strlen(capacitor->name)+2, sizeof(char));
    sprintf(nonucname, "%s_%s", tmpname, capacitor->name);
    
    return nonucname;
}

/* Concatenate a name, a number and another name */
char *
  CBLKmkName(prefix, number, postfix)
char *prefix;
int number;
char *postfix;
{
    char *finalname;
    
    finalname = (char *) calloc(strlen(prefix)+strlen(postfix)+15, sizeof(char));
    sprintf(finalname, "%s_%d_%s", prefix, number, postfix);
    
    return finalname;
}

/* place the created name under the specified directory */ 
char *
  CBLKpushDir(directory, name)
char *directory;
char *name;
{
    char *finalname;
    finalname = (char *) calloc(strlen(directory) + strlen(name) + 2, sizeof(char));
    sprintf(finalname, "%s/%s", directory, name);
    return finalname;
}
