/* among the equivalent locations for the capacitor terminal find the
one closest to the opamp or switch terminal for adore2 */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* Last Cahnge: 8/12/87 */

#include "ADglobals.h"
#include "TEKtech.h"
#include <stdio.h>

#define getsign(A) ((A) >= 0 ? 1 : -1 )

static int tpmaxshift;  /* the center of terminal can be shifted by this much in either
			   direction from the center of the unit capacitor */
static int bpmaxshift; /* same as above for bottom plate */

/*
 * CBLKexamineDataBase
 * -hcc
 */

static int
  CBLKexamineDataBase (cblk)
struct _cblk *cblk;
/* This function is called only if the DEBUG variable is set */
{
    struct _cap *cap;
    struct _terminal *term;
    struct _eqterm *eqterm;
    
    printf ("CBLKexamineDataBase: START\n");
    printf ("unitA=%d    unith=%d   unitw=%d   numrow=%d   numcol=%d\n",
	    cblk->unitA,cblk->unith,cblk->unitw,cblk->numrow,
	    cblk->numcol);
    printf ("topsum=%d   botsum=%d   totalsum=%d\n",
	    cblk->topsum,cblk->botsum,cblk->totalsum);
    printf ("ncontc=%d    nswitc=%d\n",cblk->ncontc,cblk->nswitc);
    
    printf ("Equivalent terminals (*eqtoppt)\n");
    for (term=cblk->eqtoppt;term!=NULL;term=term->next) {
	printf ("(x,y)=(%d,%d)\n",term->xcoord,term->ycoord); 
    }
    printf ("Equivalent terminals (*eqbotpt)\n");
    for (term=cblk->eqbotpt;term!=NULL;term=term->next) {
        printf ("(x,y)=(%d,%d)\n",term->xcoord,term->ycoord); 
    }
    
    printf ("Continuous time capacitors (*contcaps):\n"); 
    for (cap=cblk->contcaps;cap!=NULL;cap=cap->next) {
        printf ("tplsval=%e     bplsval=%e\n",cap->tplsval,cap->bplsval);
        printf ("terminal t:(%d,%d)  b:(%d,%d)\n",
		cap->topplate->xcoord,cap->topplate->ycoord,
		cap->botplate->xcoord,cap->botplate->ycoord);
        for (eqterm=cap->topplate->eqterms;eqterm!=NULL;eqterm=eqterm->next) {
	    printf ("Top Place EQTerm in lambda (%d,%d)\n",
		    eqterm->xcoord/20,eqterm->ycoord/20);
        }
        for (eqterm=cap->botplate->eqterms;eqterm!=NULL;eqterm=eqterm->next) {
	    printf ("Bottom Place EQTerm in lambda (%d,%d)\n",
		    eqterm->xcoord/20,eqterm->ycoord/20);
        } /* Equivalent Terminals */
    }
    
    printf ("Integrating capacitors (*integcap):\n");
    if (cblk->contcaps!=cblk->integcap) {
	for (cap=cblk->integcap;cap!=NULL;cap=cap->next) {
	    printf ("tplsval=%e     bplsval=%e\n",cap->tplsval,cap->bplsval);
	    printf ("terminal t:(%d,%d)  b:(%d,%d)\n",
		    cap->topplate->xcoord,cap->topplate->ycoord,
		    cap->botplate->xcoord,cap->botplate->ycoord);
	}
    } else {
        printf ("same as (*contcaps).\n");
    }
    
    printf ("Switched capacitors (*swcaps):\n");
    for (cap=cblk->swcaps;cap!=NULL;cap=cap->next) {
        printf ("tplsval=%e     bplsval=%e\n",cap->tplsval,cap->bplsval);
        printf ("terminal t:(%d,%d)  b:(%d,%d)\n",
		cap->topplate->xcoord,cap->topplate->ycoord,
		cap->botplate->xcoord,cap->botplate->ycoord);
        for (eqterm=cap->topplate->eqterms;eqterm!=NULL;eqterm=eqterm->next) {
	    printf ("Top Place EQTerm in lambda (%d,%d)\n",
		    eqterm->xcoord/20,eqterm->ycoord/20);
        }
        for (eqterm=cap->botplate->eqterms;eqterm!=NULL;eqterm=eqterm->next) {
	    printf ("Bottom Place EQTerm in lambda (%d,%d)\n",
		    eqterm->xcoord/20,eqterm->ycoord/20);
        } /* Equivalent Terminals */
    }
    
    /*
     * The *swcaps and *contcaps expand into the basic chunks within the
     * cblk's, not the unit capacitors.
     */ 

    printf ("CBLKexamineDataBase: END\n");  
    return 0;
}

/*
 * I know what's wrong,
 * some of the equivalent terminals were generated completely in the wrong
 * place.  This next routine will check the y coordinates of all of the
 * equivalent terminals and only keep the ones with the smallest or
 * largest value.
 * I will remove them from the linked list.  They probably shouldn't have
 * been in there in the first place. 
 * Project Presentation is in 18 hours.  This will have to do for now.
 * -hcc
 */ 

CBLKquickFix(cblk)
struct _cblk *cblk;
{
    struct _cap *cap;
    struct _eqterm *eqterm;
    struct _eqterm *first;
    int min_y,max_y; 

    DEBUG (printf ("CBLKquickFix executing...\n"));

    /* Fix the opamp types */
    for (cap=cblk->contcaps;cap!=NULL;cap=cap->next) {
        min_y = MAXINT;
        for (eqterm=cap->topplate->eqterms;eqterm!=NULL;eqterm=eqterm->next) {
           if (min_y > eqterm->ycoord) {
              min_y = eqterm->ycoord;
           } /* if */
        } /* for */
        while (cap->topplate->eqterms->ycoord > min_y) {
           cap->topplate->eqterms = cap->topplate->eqterms->next;
        } /* while */
        eqterm = cap->topplate->eqterms;
        first = eqterm;
        while ((eqterm=eqterm->next) != NULL) {
           while (eqterm->ycoord > min_y) {
              eqterm = eqterm->next;
              if (eqterm==NULL) break;
           } /* while */
           first->next = eqterm;
           if ( (first=eqterm)==NULL) break;
        } /* while */
 
        min_y = MAXINT;
        for (eqterm=cap->botplate->eqterms;eqterm!=NULL;eqterm=eqterm->next) {
           if (min_y > eqterm->ycoord) {
              min_y = eqterm->ycoord;
           } /* if */
        } /* for */
        while (cap->botplate->eqterms->ycoord > min_y) {
           cap->botplate->eqterms = cap->botplate->eqterms->next;
        } /* while */
        eqterm = cap->botplate->eqterms;
        first = eqterm;
        while ((eqterm=eqterm->next) != NULL) {
           while (eqterm->ycoord > min_y) {
              eqterm = eqterm->next;
              if (eqterm==NULL) break;
           } /* while */
           first->next = eqterm;
           if ( (first=eqterm)==NULL) break;
        } /* while */
    } /* for */

    /* Fix the switched capacitor filter types */
    for (cap=cblk->swcaps;cap!=NULL;cap=cap->next) {
        max_y = 0;
        for (eqterm=cap->topplate->eqterms;eqterm!=NULL;eqterm=eqterm->next) {
           if (max_y < eqterm->ycoord) {
              max_y = eqterm->ycoord;
           } /* if */
        } /* for */
        while (cap->topplate->eqterms->ycoord < max_y) {
           cap->topplate->eqterms = cap->topplate->eqterms->next;
        } /* while */
        eqterm = cap->topplate->eqterms;
        first = eqterm;
        while ((eqterm=eqterm->next) != NULL) {
           while (eqterm->ycoord < max_y) {
              eqterm = eqterm->next;
              if (eqterm==NULL) break;
           } /* while */
           first->next = eqterm;
           if ( (first=eqterm)==NULL) break;
        } /* while */
 
        max_y = 0;
        for (eqterm=cap->botplate->eqterms;eqterm!=NULL;eqterm=eqterm->next) {
           if (max_y < eqterm->ycoord) {
              max_y = eqterm->ycoord;
           } /* if */
        } /* for */
        while (cap->botplate->eqterms->ycoord < max_y) {
           cap->botplate->eqterms = cap->botplate->eqterms->next;
        } /* while */
        eqterm = cap->botplate->eqterms;
        first = eqterm;
        while ((eqterm=eqterm->next) != NULL) {
           while (eqterm->ycoord < max_y) {
              eqterm = eqterm->next;
              if (eqterm==NULL) break;
           } /* while */
           first->next = eqterm;
           if ( (first=eqterm)==NULL) break;
        } /* while */

    } /* for */

}

CBLKtermLoc( cblk ) 
struct _cblk *cblk;
{
    /*
     * Let's see what's in the data base at this point.
     * No Terminal Information.
     */
    DEBUG( CBLKexamineDataBase (cblk) );
    CBLKquickFix (cblk);
    DEBUG( CBLKexamineDataBase (cblk) );
    
    CBLKcaptLoc( cblk->contcaps );
    
    CBLKcaptLoc( cblk->swcaps );
    
    /*
     * Terminal Information exists relative to the cblk module.
     */
    DEBUG( CBLKexamineDataBase (cblk) );
}


CBLKcaptLoc( caplist )
struct _cap *caplist;
{
    int toppxcrd, botpxcrd;
    struct _eqterm *toppeqt, *botpeqt, *CBLK1tLoc();
    struct _cap *tmpcap;
    
    
    tmpcap = caplist;
    for (; tmpcap != NULL; tmpcap = tmpcap->next ) {
	toppeqt = CBLK1tLoc(tmpcap->topplate, tpmaxshift);
	/*** for debugging only */
	botpeqt = CBLK1tLoc(tmpcap->botplate, bpmaxshift);
	
	toppxcrd = tmpcap->topplate->xcoord;
	botpxcrd = tmpcap->botplate->xcoord;
	/*    if ( abs( toppxcrd - botpxcrd ) < minctctsp ) {
	      if ( toppeqt->xcoord != botpeqt->xcoord ) {
	      fprintf(stderr,"CBLKtermLoc;1: internal error\n");
	      exit(-1);
	      }
	      }  */
    }
    
    tmpcap = caplist;
    for (; tmpcap != NULL; tmpcap = tmpcap->next ) {
	botpeqt = CBLK1tLoc(tmpcap->botplate, bpmaxshift);
	
	toppxcrd = tmpcap->topplate->xcoord;
	botpxcrd = tmpcap->botplate->xcoord;
	if ( abs( toppxcrd - botpxcrd ) < minctctsp ) {
	    if ( toppxcrd > botpxcrd ) {
		/*** botpeqt and toppeqt have the same xcoord */
		tmpcap->topplate->xcoord = botpeqt->xcoord + tpmaxshift;
		tmpcap->botplate->xcoord = botpeqt->xcoord - bpmaxshift;
	    } else {
		tmpcap->topplate->xcoord = botpeqt->xcoord - tpmaxshift;
		tmpcap->botplate->xcoord = botpeqt->xcoord + bpmaxshift;
	    }
	}
    }
}
	
struct _eqterm *
  CBLK1tLoc( terminal, maxshift )
struct _terminal *terminal;
int maxshift; /* the maximum amount the terminal's center can be shifted
				from the center of the unit capacitor */
{
  int refxcrd;			/* reference x coordinate */
  int mindist, eqtdist;
  int shiftsign;
  struct _eqterm *tmpeqt, *opteqt;
  struct _terminal *tmpterm;


  /* each capacitor terminal connects to one opamp or switch terminal */ 
  tmpterm = terminal->netptr->firstnett;
  for ( ; tmpterm != NULL; tmpterm = tmpterm->nxtnett ) {
    if ( tmpterm->channel == terminal->channel &&
	tmpterm->module->type != CBLK ) {
      refxcrd = PERMfindXcrd(tmpterm);
      break;
    }
  }

  tmpeqt = terminal->eqterms;
  mindist = MAXINT;
  for ( ; tmpeqt != NULL; tmpeqt = tmpeqt->next ) {
    eqtdist = abs( refxcrd - PERMfindXeqt(tmpeqt));
    if ( eqtdist < mindist ) {
      mindist = eqtdist;
      opteqt = tmpeqt;
    }
  }

  if ( terminal->module->transform == NO ) {
    terminal->xcoord = opteqt->xcoord + 
      getsign(refxcrd - PERMfindXeqt(opteqt)) *min(mindist, maxshift);
  }
  else {
    terminal->xcoord = opteqt->xcoord - 
      getsign(refxcrd - PERMfindXeqt(opteqt)) *min(mindist, maxshift);
  }

  terminal->ycoord = opteqt->ycoord;

  return( opteqt );
}
