/*
 * Copyright (c) 1985 Thomas L. Quarles
 */
#ifndef LINT
static char rcsid[] =
    "INPevaluate.c $Revision: 7.1 $ on $Date: 86/03/31 11:50:26 $";
#endif LINT

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "INPdefs.h"

double
INPevaluate(line, error)
char ** line;
int *error;
{
    char * token;
    char * here;
    int mantis;
    int expo1;
    int expo2;
    int sign;
    int expsgn;
    char * tmpline;

    /* setup */
    tmpline = *line;
    *error = INPgetTok(line,&token);
    if(*error) return(0.0);
    mantis=0;
    expo1=0;
    expo2=0;
    sign=1;
    expsgn=1;
    /* loop through all of the input token */
    here = token;
    if(*here == '+') here++;    /* plus, so do nothing except skip it */
    if(*here == '-') {  /* minus, so skip it, and change sign */
        here++;
        sign = -1;
    }
    if( (*here == (char)NULL) || ((!(isdigit(*here))) && (*here != '.'))) {
        /* number looks like just a sign! */
        *error = 1;
        /* back out the 'gettok' operation */
        *line = tmpline;
        return(0);
    }
    while(isdigit(*here)) {
        /* digit, so accumulate it. */
        mantis=10*mantis+*here-'0';
        here++;
    }
    if(*here == (char) NULL) {
        /* reached the end of token - done. */
        return ((double)mantis*sign);
    }
    /* after decimal point! */
    if(*here == '.') {
        /* found a decimal point! */
        here++; /* skip to next character */
        if(*here == (char) NULL) {
            /* number ends in the decimal point */
            return ((double)mantis*sign);
        }
        while(isdigit(*here)) {
            /* digit, so accumulate it. */
            mantis=10*mantis+*here - '0';
            expo1=expo1-1;
            if(*here == (char) NULL) {
                /* reached the end of token - done. */
                return (mantis*sign*pow(10.,(double)expo1));
            }
            here++;
        }
    }
    /* now look for "E","e",etc to indicate an exponent */
    if((*here == 'E') || (*here == 'e')) {
        /* have an exponent, so skip the e */
        here++;
        /* now look for exponent sign */
        if(*here == '+') here++;    /* just skip + */
        if(*here == '-') {
            here++; /* skip over minus sign */
            expsgn = -1;    /* and make a negative exponent */
        /* now look for the digits of the exponent */
        }
        while (isdigit(*here)) {
            expo2=10*expo2+*here - '0';
            here++;
        }
    }
    /* now we have all of the numeric part of the number, time to
     * look for the scale factor (alphabetic)
     */
    switch(*here) {
        case 't':
        case 'T':
            expo1=expo1+12;
            break;
        case 'g':
        case 'G':
            expo1=expo1+9;
            break;
        case 'k':
        case 'K':
            expo1=expo1+3;
            break;
        case 'u':
        case 'U':
            expo1=expo1-6;
            break;
        case 'n':
        case 'N':
            expo1=expo1-9;
            break;
        case 'p':
        case 'P':
            expo1=expo1-12;
            break;
        case 'f':
        case 'F':
            expo1=expo1-15;
            break;
        case 'm':
        case 'M':
            {
                /* special case for m - may be m or mil or meg */
                if(*(here+1) != (char) NULL && *(here+2) != (char) NULL) {
                    /* at least 2 characters, so check them. */
                    if((*(here+1) == 'E') || (*(here+1) == 'e')) {
                        if((*(here+2) == 'G') || (*(here+2) == 'g')) {
                            expo1=expo1+6;
                            return (sign*mantis*
                                pow((double)10,(double)(expo1+expsgn*expo2)));
                        }
                    }
                    else if ((*(here+1) == 'I') || (*(here+1) == 'i')) {
                        if( (*(here+2) == 'L') || (*(here+2) == 'l')) {
                            expo1=expo1-6;
                            mantis=mantis*25.4;
                            return (sign*mantis*
                                pow((double)10,(double)(expo1+expsgn*expo2)));
                        }
                    }
                }
                /* not either special case, so just m => 1e-3 */
                expo1=expo1-3;
            }
            break;
        default:
            break;
    }
    return (sign*mantis*pow((double)10,(double)(expo1+expsgn*expo2)));
}
