/*
 * Copyright (c) 1985 Thomas L. Quarles
 */
#ifndef LINT
static char rcsid[] =
    "INPreadAll.c $Revision: 7.1 $ on $Date: 86/03/31 11:51:51 $";
#endif LINT

    /* INPreadAll(file,data) - reads the entire input file and returns
     *  a pointer to the first line of the linked list of 'card'
     *  records in data - return value is an error code (or 0).
     *
     *
     */

    /*
     * This just checks for '+' to group lines together and removes
     * lines which are comments.
     * -hcc
     */
  

#define MAXCHARPERLINE 4096

#include <stdio.h>
#include <ctype.h>
#include "utility.h"
#include "INPdefs.h"

static void
  INPcaseFix(string)
register char *string;
{
    while(*string) {
        if(isupper(*string)) {
            *string = tolower(*string);
        }
        string++;
    }
}

int
INPreadAll(file,data)
    FILE *file;
    card **data;

{
    card *head; /* current head of card list */
    card *temp; /* temporary to hold card being manipulated */
    char *linetemp; /* character string temporary */
    card *image;    /* image of line as it was read */
    card *prev;     /* previous value of image */
    char *bigbuf;   /* a large buffer to hold largest possible line */
    char *now;      /* where we are in the head input */
    int size;   /* size of line just read */
    int lineno = 1; /* line number of input */
    int c;

    bigbuf = ALLOC(char, MAXCHARPERLINE);

    head = image = ALLOC(card, 1);

    while(1) {

        now = bigbuf;
        while((c = getc(file)) != '\n' && c != EOF)
            *now++ = c;
        if(c==EOF && now == bigbuf) break;
        *now++ = '\0';
        size = now-bigbuf;
        image->line = ALLOC(char, size);
        (void) strcpy(image->line,bigbuf);
		/*
        INPcaseFix(image->line);
		*/
        image->nextcard = ALLOC(card, 1);
        image->linenum = lineno++;
        image->error = (char *)NULL;
        prev = image;
        image = image->nextcard;
    }
    /* reached EOF successfully */
    /* now throw away 'image' of 'next' line */
    FREE(prev->nextcard);
    prev->nextcard = (card *)NULL;

    /*  now to arrange these things into logical lines */
    image = head->nextcard; /* start with the second card -
                             * first is unparseable title card 
                             */
    prev = (card *)NULL;
    while(image != NULL) {
        switch(*(image->line)) {
            case '+':   /* a continuation line - splice onto previous line */
                if(prev == (card *)NULL) {
                    /* can't continue a comment or title card! */
                    image->error = ALLOC(char, 79);
                    (void) strcpy(image->error,
                            "illegal continuation card - ignored");
                    image = image->nextcard;
                    break;
                }
                if( (prev->actualLine) == (card *)NULL) {
                    /* first continuation card:  make the 'virtual' line */
                    temp = ALLOC(card, 1);
                    temp->line = prev->line;
                    temp->linenum = prev->linenum;
                    temp->error = prev->error;
                    temp->nextcard = (card *)NULL;
                    prev->actualLine = temp;
                    prev->line = ALLOC(char, strlen(temp->line)+1);
                    (void) strcpy(prev->line,temp->line);
                    prev->error = (char *)NULL;
                }
                /* adding a continuation line into the mess */
                /* prev points to the 'virtual' line */
                linetemp = ALLOC(char, strlen(prev->line)+strlen(image->line)+1);
                (void) strcpy(linetemp,prev->line);
                (void) strcat(linetemp," ");   /* a space to separate lines */
                (void) strcat(linetemp,image->line+1);  /* +1 to skip the 
                                                         * leading '+'*/
                FREE(prev->line);
                prev->line = linetemp;
                temp->nextcard = image;
                temp = image;
                prev->nextcard = image->nextcard;
                image->nextcard = (card *)NULL;
                image = prev->nextcard;
                break;
            case '\0':  /* blank line - just like a comment */
            case '*':   /* not interested in this - it's a comment */
                        /* advance to next logical line */
                        /* (make prev null so can't continue a comment) */
                prev = (card *)NULL;
                image = image->nextcard;
                break;
            default:    /* a new card - advance to next logical line */
                prev = image;
                image = image->nextcard;
                break;
        }
    }
    *data = head;
    FREE(bigbuf);
}
