
/* RCS Info: $Revision: 7.1 $ on $Date: 86/03/31 11:52:03 $
 *           $Source: /ic3/quarles/shared/INP/RCS/INPsymTab.c,v $
 * Copyright (c) 1985 Wayne A. Christopher, U. C. Berkeley CAD Group
 *
 * Stuff for the terminal and node symbol tables.
 * Defined: INPtabInit, INPlibInsert, INPtermInsert, INPtabEnd
 */

 /* last change: 7/7/87 by Hormoz Yaghuteil */


#include <stdio.h> /* Take this out soon. */
#include "utility.h"
#include "INPdefs.h"
#include "ADglobals.h"

struct tab {
        char *t_ent;
        struct tab *t_next;
		struct _lib *libptr; /* used only for libsymtab */
		struct _net *netptr; /* used only for termsymtab */
} ;

static struct tab **libsymtab, **termsymtab;
static int size, termsize, init = 0;

/* Initialize the symbol tables. */

void
INPtabInit(numlines)
{

        if (init) {
                fprintf(stderr, "Internal Error: INPtabInit called twice\n");
        } else {
                libsymtab = ALLOC(struct tab *, (numlines / 4 + 1));
                termsymtab = ALLOC(struct tab *, numlines);
                init = 1;
                size = numlines / 4 + 1;
                termsize = numlines;
        }
        return;
}

/* Insert 'token' into the terminal symbol table */
/* it also creates storage for the net if needed and returns pointer
to the found or created net */
/* changed 6/30/87 */

struct _net *INPtermInsert(token)
        char **token;
{
  int key;
  struct tab *t;
  struct _net *tmpnet;

  if (init == 0) {
    fprintf(stderr, "Internal Error: INPtabInit not called!!\n");
  } else {
    key = hash(*token, termsize);
    for (t = termsymtab[key]; t; t = t->t_next) {
      if (!strcmp(*token, t->t_ent)) {
	/* If the strings are equal */
	FREE(*token);
	*token = t->t_ent;
	return(t->netptr);
      }
    }
    t = ALLOC(struct tab, 1);
    t->t_ent = *token;
    t->t_next = termsymtab[key];
    termsymtab[key] = t;

    /* allocate memory for the new net */
    (void) INPcrtNet(&tmpnet, *token);
    t->netptr = tmpnet;
    return(t->netptr);
  }
  return( NULL ) ;
}

/* insert 'token' into the library symbol table */
/* it also creates storage for the library entry if needed and returns pointer
to the found or created library */

struct _lib *INPlibInsert(token)
        char **token;
{
        struct tab *t;
		struct _lib *tmplib;
        int key;

        if (init == 0) {
                fprintf(stderr, "Internal Error: INPtabInit not called!!\n");
        } else {
                key = hash(*token, size);
                for (t = libsymtab[key]; t; t = t->t_next)
                        if (!strcmp(*token, t->t_ent)) {
                                FREE(*token);
                                *token = t->t_ent;
                                return(t->libptr);
                        }
                t = ALLOC(struct tab, 1);
                t->t_ent = *token;
                t->t_next = libsymtab[key];
				t->netptr = NULL;
                libsymtab[key] = t;

				/* allocate memory for the new library entry */
				INPcrtLibelm(&tmplib, *token);
				t->libptr = tmplib;
				return(t->libptr);
        }
        return(NULL);
}

/* Free the space used by the symbol tables. */

void
INPtabEnd()
{
        struct tab *t, *lt;
        int i;

        if (init == 0) {
                fprintf(stderr, "Internal Error: INPtabEnd called wrongly\n");
        } else {
                for (i = 0; i < size; i++)
                        for (t = libsymtab[i]; t; t = lt) {
                                lt = t->t_next;
                                FREE(t);    /* But not t_ent ! */
                        }
                FREE(libsymtab);
                for (i = 0; i < termsize; i++)
                        for (t = termsymtab[i]; t; t = lt) {
                                lt = t->t_next;
                                FREE(t);    /* But not t_ent ! */
                        }
                FREE(termsymtab);
                init = 0;
        }
        return;
}

static
hash(name, tsize)
        char *name;
{
        char *s;
        register int i = 0;

        for (s = name; *s; s++)
                i += *s;
        return (i % tsize);
}

