/* library management for automatic switched-capacitor circuit generation */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* Last change: 7/6/87 */

/*
 * Modifications to Adore
 * Henry Chang (henryc@ic.Berkeley.EDU)
 * Fall, 1990
 */

#define C_FILE_NAME "LIBfillSw.c"

#include "ADglobals.h"
#include <stdio.h>
#include "oct.h"
#include "oh.h"

/* evaluates the dimension of the switches and the location of
opamp and capacitor ports, It also evaluates the location and width 
of the busses */

/* defined: LIBfillSw */

/*
 * Re-written: 19-January-1991 by Henry Chang
 */

LIBfillSw (libptr)
struct _lib *libptr; 			/* current switch's module */
{
    octObject           facet;                  /* Interface facet */
    octObject           term;                   /* Formal Terminal */
    octGenerator        gen;                    /* Generator for Terminals */
    octBox              box;                    /* Terminal Box */
    octStatus           gen_return;             /* Geneartor return status */
    octStatus           ohBB_return;            /* ohBB return status */
    int			x,y;

    struct _mod_stack   *mod;                   /* current module */
    struct _swch	*swch;			/* current switch */

    octObject           contents_facet;         /* for LIBgetBusses */
    octBox              facet_bb;               /* facet bounding box */

    octBegin();

    ohUnpackDefaults (&facet,"r",":physical:interface");
    ohUnpackFacetName (&facet, libptr->libname);
    facet.contents.facet.facet = "interface";
    VERBOSE (printf ("Processing the library entry, %s:%s:%s.\n",
        facet.contents.facet.cell, facet.contents.facet.view,
        facet.contents.facet.facet));
    OH_ASSERT (octOpenFacet (&facet));

    /*
     * Get Bounding Box of the Switch 
     */

    LIBgetBB (libptr,&facet,&facet_bb);

    /*
     * Get the Terminal Locations of the Switches 
     * Expects terminals on the top of the cell and on a specific layer
     */

    OH_ASSERT (octInitGenContents(&facet,OCT_TERM_MASK,&gen));
    while ( (gen_return=octGenerate (&gen, &term)) == OCT_OK) {
        ohBB_return = ohBB (&term, &box);
        if (ohBB_return != OCT_OK) {
           if (ohBB_return == OCT_NO_BB)
              errRaise (C_FILE_NAME,ohBB_return, "No Bounding Box.");
           errRaise (C_FILE_NAME,ohBB_return, "ohBB failed.");
        }
        x = (box.lowerLeft.x + box.upperRight.x ) / 2;
        y = box.upperRight.y;  /*  (box.lowerLeft.y + box.upperRight.y ) / 2; */
        /*
         * Original Program had:
         * y = box.upperRight.y
         * -hcc
         */
        if ( strcmp(term.contents.term.name,"AMP1TERM") == 0 ) {
	  DEBUG (printf ("amp1term=(%d,%d)\n",x,y));
           for (mod=libptr->stptr; mod!=NULL; mod = mod->next) {
              swch = mod->module->info.swch;
              swch->amp1term->xcoord = x;
              swch->amp1term->ycoord = y;
              swch->amp1term->chrlayer = 1;
              swch->amp1term->chloc = TOP;
              strcpy(swch->amp1term->name,term.contents.term.name);
           } /* for */
        } /* AMP1TERM */
        else if ( strcmp(term.contents.term.name,"CAPTERM") == 0 ) {
	  DEBUG (printf ("capterm=(%d,%d)\n",x,y));
           for (mod=libptr->stptr; mod!=NULL; mod = mod->next) {
              swch = mod->module->info.swch;
              swch->capterm->xcoord = x;
              swch->capterm->ycoord = y;
              swch->capterm->chrlayer = 1;
              swch->capterm->chloc = TOP;
              strcpy(swch->capterm->name,term.contents.term.name);
           } /* for */
        } /* CAPTERM */
        else if ( strcmp(term.contents.term.name,"AMP2TERM") == 0 ) {
	  DEBUG (printf ("amp2term=(%d,%d)\n",x,y));
           for (mod=libptr->stptr; mod!=NULL; mod = mod->next) {
              swch = mod->module->info.swch;
              swch->amp2term->xcoord = x;
              swch->amp2term->ycoord = y;
              swch->amp2term->chrlayer = 1;
              swch->amp2term->chloc = TOP;
              strcpy(swch->amp2term->name,term.contents.term.name);
           } /* for */
        } /* AMP2TERM */
        else if ( strcmp(term.contents.term.name,"CLK1BART") == 0 ) {
	  DEBUG (printf ("clk1bart=(%d,%d)\n",x,y));
           for (mod=libptr->stptr; mod!=NULL; mod = mod->next) {
              swch = mod->module->info.swch;
              swch->clk1bart->xcoord = x;
              swch->clk1bart->ycoord = y;
              swch->clk1bart->chrlayer = 1;
              swch->clk1bart->chloc = BOTTOM;
              strcpy(swch->clk1bart->name,term.contents.term.name);
           } /* for */
        } /* CLK1BART */
        else if ( strcmp(term.contents.term.name,"CLK1TERM") == 0 ) {
           DEBUG (printf ("clk1term=(%d,%d)\n",x,y));
           for (mod=libptr->stptr; mod!=NULL; mod = mod->next) {
              swch = mod->module->info.swch;
              swch->clk1term->xcoord = x;
              swch->clk1term->ycoord = y;
              swch->clk1term->chrlayer = 1;
              swch->clk1term->chloc = BOTTOM;
              strcpy(swch->clk1term->name,term.contents.term.name);
           } /* for */
        } /* CLK1TERM */
        else if ( strcmp(term.contents.term.name,"CLK2TERM") == 0 ) {
           DEBUG (printf ("clk2term=(%d,%d)\n",x,y));
           for (mod=libptr->stptr; mod!=NULL; mod = mod->next) {
              swch = mod->module->info.swch;
              swch->clk2term->xcoord = x;
              swch->clk2term->ycoord = y;
              swch->clk2term->chrlayer = 1;
              swch->clk2term->chloc = BOTTOM;
              strcpy(swch->clk2term->name,term.contents.term.name);
           } /* for */
        } /* CLK2TERM */
        else if ( strcmp(term.contents.term.name,"CLK2BART") == 0 ) {
           DEBUG (printf ("clk2bart=(%d,%d)\n",x,y));
           for (mod=libptr->stptr; mod!=NULL; mod = mod->next) {
              swch = mod->module->info.swch;
              swch->clk2bart->xcoord = x;
              swch->clk2bart->ycoord = y;
              swch->clk2bart->chrlayer = 1;
              swch->clk2bart->chloc = BOTTOM;
              strcpy(swch->clk2bart->name,term.contents.term.name);
           } /* for */
        } /* CLK2BART */
        else {
	  VERBOSE (printf("Non-signal Terminal, %s, found at (%d,%d).\n",
			  term.contents.term.name, x, y));
        } /* else */
    } /* while */
    octFreeGenerator (&gen);
    if (gen_return != OCT_GEN_DONE) {
        errRaise (C_FILE_NAME,0,"Did not receive OCT_GEN_DONE");
    }

   /*
    * Get Busses Bag
    * Searh for SUPPLY and GROUND nets
    * Have to look in both the contents facet and the interface facet.
    */

    ohUnpackDefaults (&contents_facet,"r",":physical:contents");
    ohUnpackFacetName (&contents_facet, libptr->libname);
    contents_facet.contents.facet.facet = "contents";
    VERBOSE (printf ("Processing the library entry, %s:%s:%s.\n",
        contents_facet.contents.facet.cell, facet.contents.facet.view,
        contents_facet.contents.facet.facet));
    OH_ASSERT (octOpenFacet (&contents_facet));

    LIBgetPowerTerms (libptr,&contents_facet,&facet,&facet_bb);
    LIBgetBusses(&contents_facet, &facet, libptr);

    OH_ASSERT (octCloseFacet (&contents_facet));
    OH_ASSERT (octCloseFacet (&facet));
    octEnd();
} /* LIBfillSw */
