/* order (switched or continuous) capacitors of a block for adore2 */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* Last Change: 8/5/87 */

#include "ADglobals.h"
#include <stdio.h>

PERMallCaps( )
{
    struct _module *tmpmod;
    struct _cblk *tmpcblk;
    struct _cap *tmpcap, *prevcap;

    for ( tmpmod = modlist[CBLK]; tmpmod != NULL; tmpmod = tmpmod->next ) {
        tmpcblk = tmpmod->info.cblk;

        PERMcapOrd( &(tmpcblk->contcaps) );
        PERMcapOrd( &(tmpcblk->swcaps) );

        /* in the master integrating capcitor always comes first */
        /* find the integrating capacitor and bring to the beginning */
        prevcap = tmpcap = tmpcblk->contcaps;
        for ( ; tmpcap != NULL; tmpcap = tmpcap->next ) {
            if ( tmpcap == tmpcblk->integcap ) {
                if ( tmpcap == tmpcblk->contcaps ) {
                    /* already in proper order */
                    break;
                }
                /* remove integcap from its current location */
                prevcap->next = tmpcap->next;
                /* put at the beginning */
                tmpcap->next = tmpcblk->contcaps;
                tmpcblk->contcaps = tmpcap;
            }
            prevcap = tmpcap;
        }
    }
}


PERMcapOrd( caplist )
struct _cap **caplist;
{
    int numcaps;
    int svalsign; /* sort in the increasing if +1 and decreasing if -1 */

    struct _cap *tmpcap;
    struct _terminal *tmpterm;


    /*** each capacitor terminal is connected to only one opamp or switch */
    /*** It might be connected to INMOD or OUTMOD */

    tmpcap = *caplist;

    if ( tmpcap->cblk->module->transform == NO ) {
        svalsign = 1;
    }
    else {
        svalsign = -1;
    }

    numcaps = 0;
    for (  ; tmpcap != NULL; tmpcap = tmpcap->next ) {
        numcaps++;

        /*** first sort acoording to topplate connections */
        tmpterm = tmpcap->topplate->netptr->firstnett;
        for ( ; tmpterm != NULL; tmpterm = tmpterm->nxtnett ) {
            if ( tmpterm->channel == tmpcap->topplate->channel &&
                                  tmpterm->module->type != CBLK ) {
                tmpcap->tplsval = svalsign * PERMfindXcrd( tmpterm );
                break;
            }
        }

        /*** next sort according to botplate connections */
        tmpterm = tmpcap->botplate->netptr->firstnett;
        for ( ; tmpterm != NULL; tmpterm = tmpterm->nxtnett ) {
            if ( tmpterm->channel == tmpcap->botplate->channel &&
                                  tmpterm->module->type != CBLK ) {
                tmpcap->bplsval = svalsign * PERMfindXcrd( tmpterm );
                break;
            }
        }
    }

    PERMbubbleSort( caplist, numcaps );
}

/* sort in the increasing order, first according to the topplate
connections, then according to the bottom plate connections */

PERMbubbleSort(caplist, numcaps)
struct _cap **caplist; /* pointer to the first element of the array */
int numcaps; /* number of capacitors to be sorted */
{
    int cntr;
    int pass; /* the pass number */
    int intchnge; /* flag to check any interchange happened the last pass */
    struct _cap *prevcap, *curcap, *tmpcap;
    int curorder, nextorder; /* the fields of the current and next elmts */


    pass = 1;
    intchnge = YES;
    while ( pass <= numcaps - 1 && intchnge == YES ) {
        intchnge = NO;
        curcap = *caplist;
        prevcap = NULL;
        for ( cntr = 1; curcap->next != NULL, cntr <= numcaps - pass; cntr++) {
            if ( curcap->tplsval > curcap->next->tplsval ||
                     (curcap->tplsval == curcap->next->tplsval &&
                            curcap->bplsval > curcap->next->bplsval)) {
                intchnge = YES;
                tmpcap = curcap->next;
                curcap->next = tmpcap->next;
                tmpcap->next = curcap;
                if ( prevcap == NULL ) {
                    *caplist = tmpcap;
                }
                else {
                    prevcap->next = tmpcap;
                }
                prevcap = tmpcap;
            }
            else {
                prevcap = curcap;
                curcap = curcap->next;
            }
        }
        pass++;
    }
}
