/* the following program generates the channel router layout for shields */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* Last Change: 8/12/87 */

/*
 * Modifications to Adore
 * Henry Chang (henryc@ic.Berkeley.EDU)
 * Fall, 1990
 */

#define C_FILE_NAME "ROUTshieldOut.c"

#include "ADglobals.h"
#include "ROUTglobals.h"
#include "TEKtech.h"
#include "oct.h"
#include <stdio.h>

ROUTshieldOut(channel,tmp_dir )
struct _channel *channel; /* the channel to be layed out */
char *tmp_dir;
{
  FILE *fp, *fopen(); /* the file information is read from */
  char newstring[128];
  int counter;
  int numvias, numwires;
  /* normalized channel coordinates */
  int chxcoord, chycoord, chx1coord, chy1coord, chx2coord, chy2coord;
  /* actual coordinates */
  int xcoord, ycoord, x1coord, y1coord, x2coord, y2coord;
  int lowleftx, lowlefty, uprightx, uprighty; /*coordinates of a box */
  int net, layer,layer1, layer2, width;
  char *layername;
  octObject Ochannel;
  char filename[FILENAME_SIZE];

  sprintf (filename, "%s/%s", tmp_dir, SH_OUT_FILE);
  fp = fopen(filename,"r");

  openfacet(&Ochannel, channel->module->name, "physical", "contents" , "a");
	
  fscanf(fp,"%s",newstring);
  while (strcmp("failed",newstring) != 0 &&
	 strcmp("channelwiring", newstring) != 0 ) {
    if(fscanf(fp,"%s",newstring) < 1) {
      strcpy(newstring, "failed");        /*  Modified by Enrico 10/30/1990 */
      break;
    }
  }
  if ( strcmp("failed",newstring) == 0 ) {
    fprintf(stderr,"shielding failed\n");
    return;
  }

  fscanf(fp,"%s %d",newstring, &numvias);
  if ( strcmp("vias",newstring ) != 0 ) {
    printf("vias should come after channel definition\n");
    exit( -1);
  }
  for ( counter = 1; counter <= numvias ; counter++ ) {
    /* read coordinates of the vias center */
    fscanf(fp,"%d %d %d %d %d",&net,&chxcoord,&chycoord,&layer1,&layer2);
  }
  if ( numvias > 0 ) {
    printf("%d number of vias not generated,(%d)\n",numvias, channel);
  }
  
  fscanf(fp,"%s %d",newstring, &numwires);
  if ( strcmp("wires",newstring ) != 0 ) {
    printf("%s\n",newstring);
    printf("wires should come after vias\n");
    exit( -1);
  }
  for ( counter = 1; counter <= numwires ; counter++ ) {
    
    fscanf(fp,"%d %d %d %d %d %d",&net, &chx1coord, &chy1coord,
	   &chx2coord, &chy2coord, &layer);
    if ( layer == 1 ) {
      /* ignore this and generate the shield as usual */
      width = polywidth;
      layername = toppoly_L;
    }
    else if ( layer == 2 ) {
      width = polywidth;
      layername = toppoly_L;
    }
    x1coord = channel->xgrids[chx1coord].xcoord;
    x2coord = channel->xgrids[chx2coord].xcoord;
    y1coord = chy1coord * minctctsp;
    y2coord = chy2coord * minctctsp;
    
    /* here I am using botextop as topexbot */
    lowleftx = min(x1coord,x2coord) - (width+botextop)/2;
    lowlefty = min(y1coord,y2coord) - (width+botextop)/2;
    uprightx = max(x1coord,x2coord) + (width+botextop) - (width+botextop)/2;
    uprighty = max(y1coord,y2coord) + (width+botextop) - (width+botextop)/2;
    
    makebox(&Ochannel,layername,lowleftx,lowlefty,uprightx,uprighty);
    
    if ( channel->name == TOP && 
	max(chy1coord,chy2coord) == channel->hichy ) { 
      /* place the contacts to the ground */
      xcoord = x1coord; /* x1coord and x2coord should be the same */
      ycoord = max(y1coord,y2coord) + max(polyovcon,metovcont) +
	mincontd/2;
      lowleftx = xcoord - mincontd / 2;
      lowlefty = ycoord - mincontd / 2;
      uprightx = lowleftx + mincontd;
      uprighty = lowlefty + mincontd;
      makebox(&Ochannel,con_L,lowleftx,lowlefty,uprightx,uprighty);
      
      /* assume polyovcon is the same as second poly ovcon */
      lowleftx -= max(polyovcon,metovcont); 
      lowlefty -= max(polyovcon,metovcont); 
      uprightx += max(polyovcon,metovcont); 
      uprighty += max(polyovcon,metovcont); 
      makebox(&Ochannel,layername,lowleftx,lowlefty,uprightx,uprighty);
    }
  }
  if ( octCloseFacet(&Ochannel) < OCT_OK) {
    octError("closing the channel facet");
    exit( -1 );
  }
  /* GenInterface (&Ochannel); */
}
