/* this reads the technology information for Adore2 */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* Last Change : Henry Chang (henryc@ic.Berkeley.EDU)  -  Fall, 1990
 *               Enrico Malavasi  -  Spring 1992 */
#define C_FILE_NAME "TEKread.c"

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "errtrap.h"
#include "utility.h"
#include "ADdef.h"
#include "ADglobals.h"
#include "INPdefs.h"
#include "TEKtech.h"

#define OBSOLETE(x)   \
  printf("Technology file: command %s is unknown or obsolete - IGNORED\n", x)

/*
 * Set these global variables to 0 to obtain a known starting condition.
 * This was not in the original program.  It should function without the
 * initializing.
 * -hcc
 */

int uctype=0;            /* type of unit capacitors top plate connection */
int shieldflag=0;        /* shielding allowed or not */

int resolution=0;         /* minimum resolution of pattern generator */
int botextop=0;           /* minimum extension of bottom plate beyond top plate*/
int toptopsp=0;           /* minimum spacing between top plates*/
int botbotsp=0;           /* minimum spacing between bottom plates*/
int ndifconsp=0;          /* spacing of ndiffusion and contact */
int pdifconsp=0;          /* spacing of pdiffusion and contact */

int topconsp=0;                               /* used for shielding only */

int selovpdif=0, selndifsp=0;              /* select layer spacing rules */
int selchsp=0; 

int oaxsep  = 0;                  /* seperation of opamps in x direction */
int blkxsep = 0;                  /* seperation of blocks in x direction */
int swxsep  = 0;                  /* seperation of switches in x direction */
int minoverhang;                  /* minimum extension of terminals beyond blocks */

int vggwidth; 
                                    /* width of vdd, vgg, and vss busses */

int Minside=0;             /* Minimum allowable side value for min capacitor */
double cap_per_ua=0;

int chipxdim=0;                                 /* dimension of the chip */

/*  This macro introduced by Enrico Malavasi
 *  10/28/1990
 */
#define   MYCONVERT(x)      ((x) * 20.0 / resolution)
#define   MYCAPCONV(c)      ((c * resolution))

/*
 * macros for this file.  One may wish to put this in TEKread.h
 * -hcc
 */
#define   TEKERROR(num,errstr)  \
	errRaise(C_FILE_NAME,0, \
        "Syntax error in line %d of Technology file:\n'%s' unknown.\n"\
	,num,errstr)

extern double INPevaluate();

TEKread(fptek) 
FILE *fptek;
{
    card *data;
    int error;
    card *current;
    char *token;
    char *line;
    char *name;
    char c;
    double val;
    char *chrval;
    int intval;
    int ctctsp1, ctctsp2, ctctsp3, ctctsp4;
    int buswidth = 0;

    polyconsp = 0;
    adTekRead();

    INPreadAll( fptek, &data );			/* Read tekfile into data */
    
    for(current = data; current != NULL; current = current->nextcard) {
	c = *(current->line);
	
	line = current->line;
	INPgetTok(&line,&name);
	
	if (c) switch(c) {
	    
	  case ' ':                     /* blank line (space leading), comment */
	  case '\t':                      /* blank line (tab leading), comment */
	  case '*':			                              /* comment */ 
	    break;
	    
	  case 'L':
	    INPgetTok(&line,&chrval);
	    if(! strcmp(name,"L_botpoly")) {
		botpoly_L = util_strsav( chrval );
	    } else if(! strcmp(name,"L_con")) {
		con_L = util_strsav( chrval );
	    } else if(! strcmp(name,"L_metal")) {
		metal_L = util_strsav( chrval );
	    } else if(! strcmp(name,"L_ndif")) {
		ndif_L = util_strsav( chrval );
	    } else if(! strcmp(name,"L_pdif")) {
		pdif_L = util_strsav( chrval );
	    } else if(! strcmp(name,"L_pwel")) {
		pwel_L = util_strsav( chrval );
	    } else if(! strcmp(name,"L_select")) {
		select_L = util_strsav( chrval );
	    } else if(! strcmp(name,"L_toppoly")) {
		toppoly_L = util_strsav( chrval );
	    } else {
		OBSOLETE( name );
	    }
	    break;
	  case 'M':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    OBSOLETE( name );
	    break;
	  case 'O':
	    INPgetTok(&line,&chrval);
	    if ( strcmp(name,"O_shield") == 0 ) {
		if ( *chrval == 'y' || *chrval == 'Y' ) {
		    shieldflag = YES;
		} else {
		    shieldflag = NO;
		}
	    } else
	      TEKERROR(current->linenum,name);
	    break;
	  case 'b':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if ( strcmp(name,"botextop") == 0 ) {
		botextop = MYCONVERT(intval);
	    } else if ( strcmp(name,"buswidth") == 0 ) {
		buswidth = MYCONVERT(intval);
	    } else
	      TEKERROR(current->linenum,name);
	    break;
	  case 'c':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if ( strcmp(name,"cap_per_ua") == 0 ) {
		cap_per_ua = MYCAPCONV(val);
	    } else {
		OBSOLETE( name );
	    }
	    break;
	  case 'm':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if ( strcmp(name,"minside") == 0 ) {
		Minside = MYCONVERT(intval);
	    } else {
		OBSOLETE( name );
	    }
	    break;
	  case 'n':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if(! strcmp(name,"ndifconsp") ) {
		ndifconsp = MYCONVERT(intval);
	    } else {
		OBSOLETE( name );
	    }
	    break;
	  case 'o':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if ( strcmp(name,"oaxsep") == 0 ) {
		oaxsep = MYCONVERT(intval);
	    } else
	      TEKERROR(current->linenum,name);
	    break;
	  case 'p':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if ( strcmp(name,"polyconsp") == 0 ) {
		polyconsp = MYCONVERT(intval);
	    } else if ( strcmp(name,"pdifconsp") == 0 ) {
		pdifconsp = MYCONVERT(intval);
	    } else {
		OBSOLETE( name );
	    }
	    break;
	  case 'r':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if ( strcmp(name,"resolution") == 0 ) {
		resolution = intval;
	    } else
	      TEKERROR(current->linenum,name);
	    break;
	  case 's':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if(! strcmp(name,"selovpdif") ) {
		selovpdif = MYCONVERT(intval);
	    }
	    else if ( strcmp(name,"selndifsp") == 0 ) {
		selndifsp = MYCONVERT(intval);
	    } else {
		OBSOLETE( name );
	    }
	    break;
	  case 't':
	    if ( strcmp(name,"topconsp") == 0 ) {
		val = INPevaluate(&line,&error);
		intval = (int) val;
		topconsp = MYCONVERT(intval);
	    } else if ( strcmp(name,"tech_name") == 0) {
		INPgetTok(&line,&chrval);
		technName = util_strsav( chrval );
	    } else
	      TEKERROR(current->linenum,name);
	    break;
	    
	  case 'u':
	    val = INPevaluate(&line,&error);
	    intval = (int) val;
	    if ( strcmp(name,"uctype") == 0 ) {
		uctype = intval;
	    } else
	      TEKERROR(current->linenum,name);
	    break;
	  default:
	    /*
	     * I've added error checking to the technology file.
	     * Comments can now only be in a certain form.  Up to now,
	     * anything that was not recognized was a comment.
	     * -hcc
	     */
	    TEKERROR(current->linenum,current->line);
	    break;
	} /* switch */
    }
    
    if (cap_per_ua == 0 ) {
	printf("TEKread: no value specified for cap_per_ua\n");
	printf("defualt value of .4 fF/micron**2 is used\n");
	cap_per_ua = MYCAPCONV(.4e-15);
    }
    cap_per_ua /= 4.0;   /* KLUDGE - waiting for new technology interface */
    
    if (resolution == 0 ) {
	printf("TEKread: no value specified for resolution\n");
	printf("defualt value of 100 units is used\n");
	resolution = 100;
    }
    if (shieldflag != YES && shieldflag != NO ) {
	printf("TEKread: no value specified for shieldflag\n");
	printf("shielding is turned off\n");
	shieldflag = NO;
    }
    if ( uctype != 1 ) {
	printf("TEKread: uctype is set to 1\n");
	uctype = 1;
    }
    if (Minside == 0 ) {
	printf("TEKread: no value specified for minside\n");
	printf("defualt value of polywidth is used\n");
	Minside = polywidth;
    }
    vggwidth = (buswidth == 0) ? metwidth : buswidth;
    
    VERBOSE (printf("TEKread: every unspecified spacing rule is set to zero.\n"));
    
    /* evluate the minimum contact to contact spacing */
    botbotsp = polypolysp; /*assume the bottom plate is the poly interconnect */
    ctctsp1 = mincontd + 2 * polyovcon + polypolysp;
    ctctsp2 = mincontd + 2 * metovcont + metmetsp; 
    ctctsp3 = metwidth + metmetsp;
    ctctsp4 = polywidth + polypolysp;
    minctctsp = max(max(ctctsp1, ctctsp2),max(ctctsp3,ctctsp4));
    
    /* evluate minimum overhang */
    /* later it should be evlauted depending on the type of extension */
    minoverhang = minctctsp - min(metwidth,polywidth);
    
    /* now evaluate the x separation of blocks */
    blkxsep = 2 * minoverhang + max(metwidth,polywidth) + minctctsp;
}
